/*******************************************************************************
 * Licensed Materials - Property of IBM
 * 
 * "Restricted Materials of IBM"
 * 
 *  Copyright IBM Corp. 2008 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * $Id:
 *******************************************************************************/

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Collections;

import org.dmtf.schemas.ovf.envelope.DocumentRoot;
import org.dmtf.schemas.ovf.envelope.EnvelopePackage;
import org.dmtf.schemas.ovf.envelope.util.EnvelopeResourceFactoryImpl;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import com.ibm.adt.ovf.model.utility.ImportVMwareAction;

/**
 * <!-- begin-user-doc -->
 * A sample utility demonstrating the generation of an OVF Envelope from a VMware VMX configuration.
 * <!-- end-user-doc -->
 */
public class VMXImportExample {
	/**
	 * <!-- begin-user-doc -->
	 * Load all the argument file paths or URIs as instances of the model.
	 * <!-- end-user-doc -->
	 * @param args
	 *        arg0 - Input VMX file to process
	 *        arg1 - Destination directory for OVF file
	 */
	public static void main(String[] args) {
		if (args.length != 2) {
			System.out.println("Incorrect number '" + args.length + "' of paramaters supplied");
			System.out.println("Input Parameters required:");
			System.out.println("Parameter 1 - Input VMX file");
			System.out.println("Parameter 2 - Output directory for generated OVF");
			return;
		}
		else {
			// First validate the input parameters
			File vmxFile = new File(args[0]);
			if (!vmxFile.isFile()) {
				System.out.println("'" + args[0] + "' is not a valid input file");
				return;
			}
			File outputDir = new File(args[1]);
			if (!outputDir.isDirectory()) {
				System.out.println("'" + args[1] + "' is not a valid output directory");
				return;
			}
			// The generated ovf will be called "inputVMXname".ovf
			String tempFileName = vmxFile.getName();
			int dotIndex = tempFileName.lastIndexOf('.');
			String ovfFileName = null;
			if (dotIndex == -1) {
				ovfFileName = tempFileName;
			} 
			else {
				// ovf file extension
				ovfFileName = tempFileName.substring(0, dotIndex) + ".ovf";
			}
			
			String outputOVFFile;
			if (outputDir.getPath().endsWith(File.separator)) {
			  outputOVFFile = outputDir.getPath() + ovfFileName;
			}
			else {
				outputOVFFile = outputDir.getPath() + File.separator + ovfFileName;
			}
			
			System.out.println("Input VMX file to process : '" + args[0] + "'");
			System.out.println("Output OVF file to be created : '" + outputOVFFile + "'");
			
			// Next, create an ImportVMwareAction utility class.  this class will parse the VMX 
			//  file and return a generated DocumentRoot.
			
			// The root element for our OVF metamodel is DocumentRoot. This is a fictitious
			// element that serves as a container for various XML things that don't 
			// really belong to the real root element, EnvelopeType.
			ImportVMwareAction importVMwareAction = new ImportVMwareAction(args[0]);

			DocumentRoot documentRoot = null;
			try {
				documentRoot = importVMwareAction.parseVmxFile();
			} catch (IOException e) {
				System.out.println("System error encountered : " + e.getMessage());
			} catch (RuntimeException e) {
				System.out.println("System error encountered : " + e.getMessage());
			} catch (URISyntaxException e) {
				System.out.println("System error encountered : " + e.getMessage());
			}

			if (documentRoot == null) {
				System.out.println("Unexpected error encountered");
				return;
			}

			// Create a resource set to hold the resource.
			//
			// A ResourceSet is where EMF holds persisted objects. As the name suggests,
			// it is a set of EMF Resources. When you load an OVF file, it will represented
			// as a Resource in a ResourceSet.
			ResourceSet resourceSet = new ResourceSetImpl();
			
			// Register the appropriate resource factory to handle all file extensions.
			// Since we are running outside of eclipse, we have to tell EMF how to 
			// associate files with models (like OVF). The DEFAULT_EXTENSION is "*" so
			// EMF will associate our Envelope stuff with any file.
			resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put
				(Resource.Factory.Registry.DEFAULT_EXTENSION, 
				 new EnvelopeResourceFactoryImpl());

			// Register the package to ensure it is available during loading.
			// Since we are running outside of eclipse, we need to register our OVF metamodel
			// with the nsURI that identifies it. For OVF, this is "http://schemas.dmtf.org/ovf/envelope/1".
			resourceSet.getPackageRegistry().put
				(EnvelopePackage.eNS_URI, 
				 EnvelopePackage.eINSTANCE);
			
			// Create a resource in our resource set. The URI is used to identify
			// the location of the file to be saved or loaded.
			Resource resource = resourceSet.createResource(URI.createFileURI(outputOVFFile));
			
			// Now add the documentRoot that the utility class created from the VMX to the resource
			resource.getContents().add(documentRoot);
			
			// Now save the OVF file
			try {
				resource.save(Collections.EMPTY_MAP); // Save with no options
			} catch (IOException e) {
				System.out.println("Error encountered saving OVF file to destination : " + e.getMessage());
			}

		}
	}
}
